<?php
/**
 * EAN for WooCommerce - Backend Class
 *
 * @version 1.3.0
 * @since   1.0.0
 * @author  Algoritmika Ltd
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_EAN_Backend' ) ) :

class Alg_WC_EAN_Backend {

	/**
	 * Constructor.
	 *
	 * @version 1.3.0
	 * @since   1.0.0
	 * @todo    [maybe] make `alg_wc_ean_backend_search_ajax` independent from `alg_wc_ean_backend_search`
	 */
	function __construct() {
		// EAN
		if ( true ) {
			$position = get_option( 'alg_wc_ean_backend_position', 'woocommerce_product_options_sku' );
			add_action( $position,           array( $this, 'add_ean_input' ) );
			add_action( 'save_post_product', array( $this, 'save_ean_input' ), 10, 2 );
			// Variations
			add_action( 'woocommerce_variation_options_pricing', array( $this, 'add_ean_input_variation' ), 10, 3 );
			add_action( 'woocommerce_save_product_variation',    array( $this, 'save_ean_input_variation' ), 10, 2 );
		}
		// Search
		if ( 'yes' === get_option( 'alg_wc_ean_backend_search', 'yes' ) ) {
			add_action( 'pre_get_posts', array( $this, 'search' ) );
			if ( 'yes' === get_option( 'alg_wc_ean_backend_search_ajax', 'yes' ) ) {
				add_filter( 'woocommerce_json_search_found_products', array( $this, 'json_search_found_products' ) );
			}
		}
		// Column
		if ( 'yes' === get_option( 'alg_wc_ean_backend_column', 'yes' ) ) {
			add_filter( 'manage_edit-product_columns',        array( $this, 'add_product_columns' ) );
			add_action( 'manage_product_posts_custom_column', array( $this, 'render_product_columns' ) );
		}
		// Backend loaded
		do_action( 'alg_wc_ean_backend_loaded', $this );
	}

	/**
	 * json_search_found_products.
	 *
	 * @version 1.0.2
	 * @since   1.0.2
	 * @todo    [maybe] customizable `meta_compare` (can be e.g. `=`)
	 * @todo    [maybe] append product_id to the title
	 */
	function json_search_found_products( $products ) {
		if ( isset( $_REQUEST['term'] ) && '' !== $_REQUEST['term'] ) {
			$key = alg_wc_ean()->core->ean_key;
			$found_products = wc_get_products( array(
				'type'         => array_merge( array_keys( wc_get_product_types() ), array( 'variation' ) ),
				'limit'        => -1,
				'meta_key'     => $key,
				'meta_value'   => wc_clean( $_REQUEST['term'] ),
				'meta_compare' => 'LIKE',
				'return'       => 'ids',
			) );
			foreach ( $found_products as $product_id ) {
				$ean = sprintf( __( 'EAN: %s', 'ean-for-woocommerce' ), get_post_meta( $product_id, $key, true ) );
				$products[ $product_id ] = get_the_title( $product_id ) . ' (' . $ean . ')';
			}
		}
		return $products;
	}

	/**
	 * add_product_columns.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function add_product_columns( $columns ) {
		$_columns = array();
		foreach ( $columns as $column_key => $column_title ) {
			$_columns[ $column_key ] = $column_title;
			if ( 'sku' === $column_key ) {
				$_columns['ean'] = __( 'EAN', 'ean-for-woocommerce' );
			}
		}
		return $_columns;
	}

	/**
	 * render_product_column_ean.
	 *
	 * @version 1.0.1
	 * @since   1.0.1
	 */
	function render_product_column_ean( $do_validate, $ean ) {
		return ( $do_validate && ! alg_wc_ean()->core->is_valid_ean( $ean ) ? '<span style="color:red;">' . $ean . '</span>' : $ean );
	}

	/**
	 * render_product_columns.
	 *
	 * @version 1.0.1
	 * @since   1.0.0
	 * @todo    [next] sortable
	 */
	function render_product_columns( $column ) {
		if ( 'ean' === $column ) {
			$product_id  = get_the_ID();
			$product     = wc_get_product( $product_id );
			$values      = array();
			$do_validate = get_option( 'alg_wc_ean_backend_column_validate', 'no' );
			if ( '' != ( $value = alg_wc_ean()->core->get_ean( $product_id ) ) ) {
				$values[] = $this->render_product_column_ean( $do_validate, $value );
			}
			if ( $product->is_type( 'variable' ) ) {
				foreach ( $product->get_children() as $child_id ) {
					if ( '' != ( $value = alg_wc_ean()->core->get_ean( $child_id ) ) ) {
						$values[] = $this->render_product_column_ean( $do_validate, $value );
					}
				}
			}
			if ( ! empty( $values ) ) {
				echo implode( ', ', $values );
			}
		}
	}

	/**
	 * search.
	 *
	 * @version 1.0.3
	 * @since   1.0.0
	 * @todo    [maybe] rewrite?
	 */
	function search( $query ) {
		if ( $query->is_main_query() && isset( $query->query['post_type'] ) && 'product' == $query->query['post_type'] ) {
			$new_query   = clone( $query );
			$search_term = isset( $_REQUEST['s'] ) ? sanitize_text_field( $_REQUEST['s'] ) : '';
			if ( empty( $search_term ) ) {
				return;
			}

			$new_query->query_vars['s'] = '';
			$old_product_in = $query->query_vars['post__in'];

			unset( $new_query->query['post__in'] );
			unset( $new_query->query_vars['post__in'] );

			$new_meta_query = array(
				'key'     => alg_wc_ean()->core->ean_key,
				'value'   => $search_term,
				'compare' => 'LIKE'
			);
			$old_meta_query = ( isset( $query->query_vars['meta_query'] ) ? $query->query_vars['meta_query'] : false );
			if ( ! empty( $old_meta_query ) ) {
				$meta_query = $old_meta_query;
				array_push( $meta_query, array( 'relation' => 'OR' ) );
				array_push( $meta_query, $new_meta_query );
			} else {
				$meta_query = array( $new_meta_query );
			}
			$new_query->set( 'meta_query', $meta_query );
			$new_query->set( 'fields', 'ids' );

			remove_action( 'pre_get_posts', array( $this, 'search' ) );

			// Search for products
			$result  = get_posts( $new_query->query_vars );
			$new_ids = $old_product_in;
			if ( $result ) {
				$new_ids = array_merge( $new_ids, $result );
			}

			// Search for variation
			$new_query->set( 'post_type', 'product_variation' );
			$new_query->set( 'fields', 'id=>parent' );
			$result = get_posts( $new_query->query_vars );
			if ( $result ) {
				$new_ids = array_merge( $new_ids, $result );
			}

			$query->set( 'post__in', $new_ids );
		}
	}

	/**
	 * get_ean_input_desc.
	 *
	 * @version 1.0.1
	 * @since   1.0.1
	 */
	function get_ean_input_desc( $ean ) {
		return ( alg_wc_ean()->core->is_valid_ean( $ean ) ?
			'<span style="color:green;">' . esc_html__( 'Valid EAN', 'ean-for-woocommerce' )   . '</span>' :
			'<span style="color:red;">'   . esc_html__( 'Invalid EAN', 'ean-for-woocommerce' ) . '</span>' );
	}

	/**
	 * get_ean_input_pattern.
	 *
	 * @version 1.0.1
	 * @since   1.0.1
	 * @todo    [maybe] `array( 'pattern' => '.{0}|[0-9]{13}', 'maxlength' => '13' ) )`
	 * @todo    [maybe] `array( 'pattern' => '.{0}|[0-9]+', 'minlength' => '13', 'maxlength' => '13' )`
	 */
	function get_ean_input_pattern( $atts = array() ) {
		return array_merge( $atts, array( 'pattern' => '[0-9]+', 'maxlength' => '13' ) );
	}

	/**
	 * add_ean_input_variation.
	 *
	 * @version 1.0.1
	 * @since   1.0.0
	 */
	function add_ean_input_variation( $loop, $variation_data, $variation ) {
		$key = alg_wc_ean()->core->ean_key;
		woocommerce_wp_text_input( array(
			'id'                => "variable{$key}_{$loop}",
			'name'              => "variable{$key}[{$loop}]",
			'value'             => ( isset( $variation_data[ $key ][0] ) ? $variation_data[ $key ][0] : '' ),
			'label'             => __( 'EAN', 'ean-for-woocommerce' ),
			'wrapper_class'     => 'form-row form-row-full',
			'placeholder'       => alg_wc_ean()->core->get_ean( $variation->post_parent ),
			'description'       => ( ! empty( $variation_data[ $key ][0] ) ? $this->get_ean_input_desc( $variation_data[ $key ][0] ) : '' ),
			'custom_attributes' => $this->get_ean_input_pattern(),
		) );
	}

	/**
	 * save_ean_input_variation.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function save_ean_input_variation( $variation_id, $i ) {
		$key = alg_wc_ean()->core->ean_key;
		if ( isset( $_POST[ 'variable' . $key ][ $i ] ) ) {
			update_post_meta( $variation_id, $key, wc_clean( $_POST[ 'variable' . $key ][ $i ] ) );
		}
	}

	/**
	 * add_ean_input.
	 *
	 * @version 1.0.1
	 * @since   1.0.0

	 */
	function add_ean_input() {
		$value = alg_wc_ean()->core->get_ean();
		woocommerce_wp_text_input( array(
			'id'                => alg_wc_ean()->core->ean_key,
			'value'             => $value,
			'label'             => esc_html__( 'EAN', 'ean-for-woocommerce' ),
			'description'       => ( ! empty( $value ) ? $this->get_ean_input_desc( $value ) : '' ),
			'custom_attributes' => $this->get_ean_input_pattern(),
		) );
	}

	/**
	 * save_ean_input.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [maybe] save `$key . '_is_valid'` (same in `save_ean_input_variation()`)
	 */
	function save_ean_input( $post_id, $__post ) {
		if ( isset( $_POST[ alg_wc_ean()->core->ean_key ] ) && empty( $_REQUEST['woocommerce_quick_edit'] ) ) {
			update_post_meta( $post_id, alg_wc_ean()->core->ean_key, wc_clean( $_POST[ alg_wc_ean()->core->ean_key ] ) );
		}
	}

}

endif;

return new Alg_WC_EAN_Backend();
