<?php
/**
 * EAN for WooCommerce - Frontend Class
 *
 * @version 1.3.0
 * @since   1.0.0
 * @author  Algoritmika Ltd
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_EAN_Frontend' ) ) :

class Alg_WC_EAN_Frontend {

	/**
	 * Constructor.
	 *
	 * @version 1.3.0
	 * @since   1.0.0
	 * @todo    [next] add more standards (i.e. not only EAN-13)
	 * @todo    [later] shortcode
	 * @todo    [later] loop
	 * @todo    [later] cart
	 * @todo    [later] order
	 */
	function __construct() {
		// EAN
		if ( 'yes' === get_option( 'alg_wc_ean_frontend', 'yes' ) ) {
			add_action( 'woocommerce_product_meta_start', array( $this, 'add_ean' ) );
			// Variations
			add_action( 'wp_enqueue_scripts',              array( $this, 'variations_enqueue_scripts' ) );
			add_filter( 'woocommerce_available_variation', array( $this, 'variations_add_params' ), 10, 3 );
		}
		// Search
		if ( 'yes' === get_option( 'alg_wc_ean_frontend_search', 'yes' ) ) {
			add_action( 'pre_get_posts', array( $this, 'search' ), 10 );
		}
		// Product structured data
		if ( 'yes' === get_option( 'alg_wc_ean_frontend_product_structured_data', 'yes' ) ) {
			add_filter( 'woocommerce_structured_data_product', array( $this, 'add_ean_to_product_structured_data' ), 10, 2 );
		}
		// Frontend loaded
		do_action( 'alg_wc_ean_frontend_loaded', $this );
	}

	/**
	 * add_ean_to_product_structured_data.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @see     http://schema.org/Product
	 */
	function add_ean_to_product_structured_data( $markup, $product ) {
		if ( '' !== ( $value = alg_wc_ean()->core->get_ean( $product->get_id() ) ) ) {
			$markup['gtin13'] = $value;
		}
		return $markup;
	}

	/**
	 * variations_enqueue_scripts.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 */
	function variations_enqueue_scripts() {
		wp_enqueue_script( 'alg-wc-ean-variations',
			alg_wc_ean()->plugin_url() . '/includes/js/alg-wc-ean-variations' . ( defined( 'WP_DEBUG' ) && true === WP_DEBUG ? '' : '.min' ) . '.js',
			array( 'jquery' ),
			alg_wc_ean()->version,
			true
		);
	}

	/**
	 * variations_add_params.
	 *
	 * @version 1.3.0
	 * @since   1.0.0
	 */
	function variations_add_params( $args, $product, $variation ) {
		$key = alg_wc_ean()->core->ean_key;
		$args['ean'] = $variation->get_meta( $key );
		return $args;
	}

	/**
	 * search.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [maybe] rewrite?
	 */
	function search( $wp_query  ) {
		global $wpdb;

		if ( ! isset( $wp_query->query['s'] ) ) {
			return;
		}

		$key   = alg_wc_ean()->core->ean_key;
		$posts = $wpdb->get_col( $wpdb->prepare( "SELECT post_id FROM {$wpdb->postmeta} WHERE meta_key='{$key}' AND meta_value LIKE %s;",
			esc_sql( '%' . $wp_query->query['s'] . '%' ) ) );
		if ( ! $posts ) {
			return;
		}

		unset( $wp_query->query['s'] );
		unset( $wp_query->query_vars['s'] );
		$wp_query->query['post__in'] = array();
		foreach($posts as $id){
			$post = get_post($id);
			if ( $post->post_type == 'product_variation' ) {
				$wp_query->query['post__in'][] = $post->post_parent;
				$wp_query->query_vars['post__in'][] = $post->post_parent;
			} else {
				$wp_query->query_vars['post__in'][] = $post->ID;
			}
		}
	}

	/**
	 * get_ean_output_data.
	 *
	 * @version 1.1.0
	 * @since   1.1.0
	 * @todo    [next] better solution for variable products
	 */
	function get_ean_output_data( $check_if_valid = false ) {
		$result = array( 'do_output' => false, 'style' => '', 'value' => alg_wc_ean()->core->get_ean() );
		if ( '' !== $result['value'] && ( ! $check_if_valid || alg_wc_ean()->core->is_valid_ean( $result['value'] ) ) ) {
			$result['do_output'] = true;
		} else {
			global $product;
			if ( $product && $product->is_type( 'variable' ) ) {
				$result['do_output'] = true;
				$result['style']     = ' style="display:none;"';
			}
		}
		return $result;
	}

	/**
	 * add_ean.
	 *
	 * @version 1.2.0
	 * @since   1.0.0
	 * @todo    [maybe] `esc_html__( 'N/A', 'ean-for-woocommerce' )`
	 */
	function add_ean() {
		$output_data = $this->get_ean_output_data();
		if ( $output_data['do_output'] ) {
			echo '<span class="sku_wrapper ean_wrapper"' . $output_data['style'] . '>' . esc_html__( 'EAN:', 'ean-for-woocommerce' ) . ' ' .
				'<span class="ean">' . $output_data['value'] . '</span>' .
			'</span>';
		}
	}

}

endif;

return new Alg_WC_EAN_Frontend();
