<?php
/**
 * EAN for WooCommerce - General Section Settings
 *
 * @version 1.3.0
 * @since   1.0.0
 * @author  Algoritmika Ltd
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_EAN_Settings_General' ) ) :

class Alg_WC_EAN_Settings_General extends Alg_WC_EAN_Settings_Section {

	/**
	 * Constructor.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function __construct() {
		$this->id   = '';
		$this->desc = __( 'General', 'ean-for-woocommerce' );
		parent::__construct();
	}

	/**
	 * get_settings.
	 *
	 * @version 1.3.0
	 * @since   1.0.0
	 * @todo    [maybe] `alg_wc_ean_backend_position`: add more positions
	 */
	function get_settings() {

		$plugin_settings = array(
			array(
				'title'    => __( 'EAN Options', 'ean-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_ean_plugin_options',
			),
			array(
				'title'    => __( 'EAN', 'ean-for-woocommerce' ),
				'desc'     => '<strong>' . __( 'Enable plugin', 'ean-for-woocommerce' ) . '</strong>',
				'id'       => 'alg_wc_ean_plugin_enabled',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_ean_plugin_options',
			),
		);

		$backend_settings = array(
			array(
				'title'    => __( 'Backend Options', 'ean-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_ean_backend_options',
			),
			array(
				'title'    => __( 'Position', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'Set to which product data tab EAN field should be added.', 'ean-for-woocommerce' ),
				'id'       => 'alg_wc_ean_backend_position',
				'default'  => 'woocommerce_product_options_sku',
				'type'     => 'select',
				'class'    => 'chosen_select',
				'options'  => array(
					'woocommerce_product_options_general_product_data'   => __( 'General', 'ean-for-woocommerce' ),
					'woocommerce_product_options_inventory_product_data' => __( 'Inventory', 'ean-for-woocommerce' ),
					'woocommerce_product_options_sku'                    => __( 'Inventory: SKU', 'ean-for-woocommerce' ),
					'woocommerce_product_options_advanced'               => __( 'Advanced', 'ean-for-woocommerce' ),
				),
			),
			array(
				'title'    => __( 'Search', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'This will enable searching by EAN in admin area.', 'ean-for-woocommerce' ),
				'id'       => 'alg_wc_ean_backend_search',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'desc'     => __( 'AJAX search', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'This will enable searching by EAN in AJAX.', 'ean-for-woocommerce' ) . ' ' .
					__( 'E.g. when searching for a product when creating new order in admin area.', 'ean-for-woocommerce' ),
				'id'       => 'alg_wc_ean_backend_search_ajax',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Product list column', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'desc_tip' => sprintf( __( 'This will add "EAN" column to %s.', 'ean-for-woocommerce' ),
					'<a href="' . admin_url( 'edit.php?post_type=product' ) . '">' . __( 'admin products list', 'ean-for-woocommerce' ) . '</a>' ),
				'id'       => 'alg_wc_ean_backend_column',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'desc'     => __( 'Validate', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'Validate EAN in column.', 'ean-for-woocommerce' ) . ' ' . __( 'Invalid EANs will be marked red.', 'ean-for-woocommerce' ),
				'id'       => 'alg_wc_ean_backend_column_validate',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Show barcode', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'This will show barcode image on admin product edit page.', 'ean-for-woocommerce' ) .
					apply_filters( 'alg_wc_ean_settings', '<br>' . 'You will need <a target="_blank" href="https://wpfactory.com/item/ean-for-woocommerce/">EAN for WooCommerce Pro</a> to enable this option.' ),
				'id'       => 'alg_wc_ean_backend_barcode',
				'default'  => 'no',
				'type'     => 'checkbox',
				'custom_attributes' => apply_filters( 'alg_wc_ean_settings', array( 'disabled' => 'disabled' ) ),
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_ean_backend_options',
			),
		);

		$frontend_settings = array(
			array(
				'title'    => __( 'Frontend Options', 'ean-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_ean_frontend_options',
			),
			array(
				'title'    => __( 'Show EAN', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'This will show EAN on single product page on frontend.', 'ean-for-woocommerce' ),
				'id'       => 'alg_wc_ean_frontend',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Search', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'This will enable searching by EAN on frontend.', 'ean-for-woocommerce' ),
				'id'       => 'alg_wc_ean_frontend_search',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Product structured data', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'id'       => 'alg_wc_ean_frontend_product_structured_data',
				'desc_tip' => __( 'This will add EAN to the product structured data, e.g. for Google Search Console.', 'ean-for-woocommerce' ),
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Show barcode', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'This will show barcode image on single product page on frontend.', 'ean-for-woocommerce' ) .
					apply_filters( 'alg_wc_ean_settings', '<br>' . 'You will need <a target="_blank" href="https://wpfactory.com/item/ean-for-woocommerce/">EAN for WooCommerce Pro</a> to enable this option.' ),
				'id'       => 'alg_wc_ean_frontend_barcode',
				'default'  => 'no',
				'type'     => 'checkbox',
				'custom_attributes' => apply_filters( 'alg_wc_ean_settings', array( 'disabled' => 'disabled' ) ),
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_ean_frontend_options',
			),
		);

		$order_items_table_settings = array(
			array(
				'title'    => __( 'Order Items Table Options', 'ean-for-woocommerce' ),
				'desc'     => __( 'This will affect all places where order items table is displayed, including <strong>emails</strong> (both admin and customer), <strong>"thank you" page</strong> (i.e. "order received" page), <strong>"view order" page</strong> (in "my account")).', 'ean-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_ean_order_items_table_options',
			),
			array(
				'title'    => __( 'Show EAN', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'This will show EAN in order items table.', 'ean-for-woocommerce' ),
				'id'       => 'alg_wc_ean_order_items_table',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Show barcode', 'ean-for-woocommerce' ),
				'desc'     => __( 'Enable', 'ean-for-woocommerce' ),
				'desc_tip' => __( 'This will show barcode image in order items table.', 'ean-for-woocommerce' ) . ' ' .
					__( '<strong>Please note</strong> that this option won\'t work on <code>localhost</code> environment.', 'ean-for-woocommerce' ) .
					apply_filters( 'alg_wc_ean_settings', '<br>' . 'You will need <a target="_blank" href="https://wpfactory.com/item/ean-for-woocommerce/">EAN for WooCommerce Pro</a> to enable this option.' ),
				'id'       => 'alg_wc_ean_order_items_table_barcode',
				'default'  => 'no',
				'type'     => 'checkbox',
				'custom_attributes' => apply_filters( 'alg_wc_ean_settings', array( 'disabled' => 'disabled' ) ),
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_ean_order_items_table_options',
			),
		);

		return array_merge( $plugin_settings, $backend_settings, $frontend_settings, $order_items_table_settings );
	}

}

endif;

return new Alg_WC_EAN_Settings_General();
